/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */


#include "Document.h"
#include "Config.h"
#include <iostream>
#include "utility.h"

#include "SubtitleSystem.h"

#include <ctime>

/*
 *	constructeur
 *	s'il y a une option dans le fichier config [encodings] default=xxxx
 *	on l'utilise pour choisir le charset, sinon on utilise UTF-8
 */
Document::Document()
{
	Config &cfg = Config::getInstance();
	Glib::ustring encoding;
	
	if(cfg.get_value_string("encodings", "default", encoding))
		m_charset = encoding;
	else
		m_charset = "UTF-8";
#warning "Fixme > Use Config Option"
	m_format = "SubRip";

	m_newline = "Unix";

	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);

	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	//m_nameModel = Glib::RefPtr<NameModel>(new NameModel);
	
	m_subtitleView = manage(new SubtitleView(*this));

	m_subtitleView->show();
}

/*
 *	constructeur par copy
 */
Document::Document(Document &src)
{
	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);
	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	g_return_if_fail(m_subtitleModel);
	g_return_if_fail(m_styleModel);

	m_format = src.m_format;
	m_charset = src.m_charset;
	m_newline = src.m_newline;

	m_scriptInfo = src.m_scriptInfo;
	
	m_name = src.m_name;
	m_filename = src.m_filename;

	m_uriMovie = src.m_uriMovie;
	
	// model
	m_subtitleModel->copy(src.get_subtitle_model());

	m_styleModel->copy(src.get_style_model());

	// subtitle view
	m_subtitleView = manage(new SubtitleView(*this));

	m_subtitleView->show();

	m_data = src.m_data;
}

/*
 *	destructeur
 */
Document::~Document()
{
}

/*
 *	le nom complet du document 
 *	ex: /home/toto/subtitle05.ass
 */
void Document::setFilename(const Glib::ustring &filename)
{
	m_filename = filename;

	m_name = Glib::path_get_basename(m_filename);

	emit_signal("document-property-changed");
}

/*
 *	le nom complet du document 
 *	ex: /home/toto/subtitle05.ass
 */
Glib::ustring Document::getFilename()
{
	return m_filename;
}

/*
 *	le nom du document
 *	si filename = /home/toto/subtitle05.ass
 *	name = subtitle05.ass
 */
void Document::setName(const Glib::ustring &name)
{
	m_name = name;

	emit_signal("document-property-changed");
}

/*
 *	le nom du document
 *	si filename = /home/toto/subtitle05.ass
 *	name = subtitle05.ass
 */
Glib::ustring Document::getName()
{
	return m_name;
}

/*
 *
 */
void Document::setFormat(const Glib::ustring &format)
{
	m_format = format;
}

/*
 *
 */
Glib::ustring Document::getFormat()
{
	return m_format;
}

/*
 *
 */
void Document::setCharset(const Glib::ustring &charset)
{
	m_charset = charset;
}

/*
 *	retourne le charset (encodage) du document
 *	a permet de sauvegarder dans le meme encodage qu'a la lecture
 *	...
 */
Glib::ustring Document::getCharset()
{
	return m_charset;
}

/*
 *	"Unix", "Windows"
 */
void Document::setNewLine(const Glib::ustring &name)
{
	m_newline = name;
}

/*
 *	"Unix", "Windows"
 */
Glib::ustring Document::getNewLine()
{
	return m_newline;
}

/*
 *	efface les informations :
 *	 - script info
 *	 - subtitle model
 *	 - style model
 *	
 *	reinitialize le charset
 */
void Document::clear()
{
	Config &cfg = Config::getInstance();
	Glib::ustring encoding;

	if(cfg.get_value_string("encodings", "default", encoding))
		m_charset = encoding;
	else
		m_charset = "UTF-8";

	//m_format = ASS;
	
	m_scriptInfo = ScriptInfo();
	
	m_filename.clear();
	m_uriMovie.clear();
	
	m_subtitleModel->clear();
	m_styleModel->clear();
	//m_nameModel->clear();
}

/*
 *
 */
bool Document::open(const Glib::ustring &_filename)
{
	/*
	//SE::getInstance()->getSubtitleView()->unset_model();

	// 29
	//
	std::clock_t start = std::clock();

	bool res = _open(filename, encoding, clear_doc);

	std::cout << "timer: " << double(std::clock() - start) / CLOCKS_PER_SEC << std::endl;

	//SE::getInstance()->getSubtitleView()->set_model(m_subtitleModel);

	emit_signal("document-property-changed");

	return res;
	*/

#define DIALOG_ERROR(ex, filename, charset) \
	if(charset.empty())	{ \
			dialog_error( \
					build_message( \
						_("Could not open the file <i>%s</i>."), \
						filename.c_str()), \
					ex.what()); } \
	else { \
			dialog_error( \
					build_message( \
						_("Could not open the file <i>%s</i> using the <i>%s</i> character coding."), \
						filename.c_str(), charset.c_str()), \
					ex.what());	}

	Glib::ustring filename = _filename;
	Glib::ustring charset = getCharset();

	try
	{
		Glib::ustring format = SubtitleSystem::getInstance().find_subtitle_format(filename);

		if(!format.empty())
		{
			std::auto_ptr<SubtitleFormat> sf(SubtitleSystem::getInstance().create_subtitle_format(format, this));

			if(sf.get() != NULL) // need ?
			{
				bool res = sf->open(filename);

				if(res)
				{
					setFilename(filename);
					setCharset(sf->get_charset());
					setFormat(format);

					emit_signal("document-property-changed");
				}

				return res;
			}
		}
	}
	catch(const SubtitleException &ex)
	{
		DIALOG_ERROR(ex, filename, charset);
	}
	catch(const Glib::Error &ex)
	{
		DIALOG_ERROR(ex, filename, charset);
	}
	catch(const std::exception &ex)
	{
		DIALOG_ERROR(ex, filename, charset);
	}

#undef DIALOG_ERROR
	return false;
}


/*
 *
 */
bool Document::_open(const Glib::ustring &filename, const Glib::ustring &encoding, bool clear_doc)
{
	if(clear_doc)
		clear();
	
	SubtitleFormat* sub = NULL;
	
	try
	{
		Glib::ustring format = SubtitleSystem::getInstance().find_subtitle_format(filename);

		if(!format.empty())
		{
			SubtitleFormat* sub = SubtitleSystem::getInstance().create_subtitle_format(format, this);

			if(sub)
			{
				bool res = sub->open(filename, encoding);

				if(res && clear_doc)
				{
					setFilename(filename);
					m_charset = sub->get_charset();

					if(m_charset.empty())
						setCharset("UTF-8");

					m_format = format;
				}

				delete sub;

				return res;
			}
		}
		else
			;//throw
	}
	catch(const SubtitleException &ex)
	{
		delete sub;

		if(encoding.empty())
		{
			dialog_error(
					build_message(
						_("Could not open the file <i>%s</i>."), 
						filename.c_str()), 
					ex.what());
		}
		else
		{
			dialog_error(
					build_message(
						_("Could not open the file <i>%s</i> using the <i>%s</i> character coding."), 
						filename.c_str(), encoding.c_str()), 
					ex.what());
		}
	}
	catch(const Glib::ConvertError &ex)
	{
		delete sub;

		if(encoding.empty())
		{
			dialog_error(
					build_message(
						_("Could not open the file <i>%s</i>."), 
						filename.c_str()), 
					ex.what());
		}
		else
		{
			dialog_error(
					build_message(
						_("Could not open the file <i>%s</i> using the <i>%s</i> character coding."), 
						filename.c_str(), encoding.c_str()), 
					ex.what());
		}
	}

	return false;

}

/*
 *	encodings ""=auto, "UTF-8", ...
 *	format "SubRip", "MicroDVD", "MPL2", ...
 *	rename, change le nom du document (filename)
 */
bool Document::save(const Glib::ustring &_filename)
{
	/*
	if(_save(filename, format, encoding, clear_doc))
	{
		if(clear_doc)
			emit_signal("document-property-changed");
			
		return true;
	}
	*/

	try
	{
		Glib::ustring filename = _filename;
		Glib::ustring format = getFormat();
		Glib::ustring charset = getCharset();

		std::auto_ptr<SubtitleFormat> sf(SubtitleSystem::getInstance().create_subtitle_format(format, this));
		
		if(sf.get() != NULL) // need ?
		{
			bool res = sf->save(filename);

			if(res)
			{
				setCharset(charset);
				setFilename(filename);
				setFormat(format);

				emit_signal("document-property-changed");
			}

			return res;
		}
	}
	catch(const SubtitleException &ex)
	{
		dialog_error(_("Save Document Failed."), ex.what());
	}
	catch(const std::exception &ex)
	{
		dialog_error(_("Save Document Failed."), ex.what());
	}
	return false;
}

/*
 *
 */
bool Document::_save(const Glib::ustring &filename, const Glib::ustring &format, const Glib::ustring &encoding, bool rename)
{
	Glib::ustring charset=encoding;

	//
	if(!charset.empty() && rename)
		m_charset = charset;
	//
	if(encoding.empty())
		charset = getCharset();

	if(rename)
	{
		setFilename(filename);
		
		m_format = format;
	}

	SubtitleFormat *subtitle = NULL;
	
	try
	{
		subtitle = SubtitleSystem::getInstance().create_subtitle_format(format, this);
		if(subtitle != NULL)
		{
			bool state = subtitle->save(filename, charset);

			delete subtitle;

			return state;
		}
	}
	catch(const SubtitleException &ex)
	{
		delete subtitle;
		dialog_error(_("Save Document Failed."), ex.what());
	}

	return false;
}

/*
 *	retourne le model pour la gestion des sous-titres
 *	on passe par un model (gtk) pour eviter d'avoir des doublons 
 *	dans les donnes et donc une grosse consomation memoire (inutile)
 */
Glib::RefPtr<SubtitleModel> Document::get_subtitle_model()
{
	return m_subtitleModel;
}

/*
 *	retourne le model pour la gestion des styles
 */
Glib::RefPtr<StyleModel> Document::get_style_model()
{
	return m_styleModel;
}

/*
 *	retourne les informations sur le script 
 *	principalement pour SSA/ASS
 */
ScriptInfo& Document::get_script_info()
{
	return m_scriptInfo;
}

/*
 *
 */
SubtitleView* Document::get_subtitle_view()
{
	return m_subtitleView;
}

/*
 *	affiche un message pour l'utilisateur sur ce document
 *	statusbar
 */
void Document::message(const gchar *format, ...)
{
	va_list args;
	gchar *formatted = NULL;

	va_start(args, format);
	formatted = g_strdup_vprintf(format, args);
	va_end(args);

	se_debug_message(SE_DEBUG_APP, formatted);

	m_signal_message(formatted);
	
	g_free(formatted);
}

/*
 *
 */
sigc::signal<void, Glib::ustring>& Document::get_signal_message()
{
	return m_signal_message;
}


/*
 *	affiche un message pour l'utilisateur sur ce document
 *	statusbar
 */
void Document::flash_message(const gchar *format, ...)
{
	va_list args;
	gchar *formatted = NULL;

	va_start(args, format);
	formatted = g_strdup_vprintf(format, args);
	va_end(args);

	se_debug_message(SE_DEBUG_APP, formatted);

	m_signal_flash_message(formatted);

	g_free(formatted);
}

/*
 *
 */
sigc::signal<void, Glib::ustring>& Document::get_signal_flash_message()
{
	return m_signal_flash_message;
}



/*
 *
 */
sigc::signal<void>& Document::get_signal(const std::string &name)
{
//	std::map<std::string, sigc::signal<void> >::iterator it =	m_signal.find(name);
//	if(it != m_signal.end())
//		return it->second;
	return m_signal[name];
}

void Document::emit_signal(const std::string &name)
{
	m_signal[name].emit();
}

/*
 *
 */
void Document::set_data(const Glib::ustring &key, const Glib::ustring &value)
{
	m_data[key] = value;
}

/*
 *
 */
Glib::ustring Document::get_data(const Glib::ustring &key)
{
	std::map<Glib::ustring, Glib::ustring>::iterator it = m_data.find(key);

	if(it == m_data.end())
	{
		std::cerr << build_message("get_data failed:'%s'", key.c_str()) << std::endl;
		return "";
	}

	return it->second;
}

/*
 *
 */
bool Document::has_data(const Glib::ustring &key)
{
	std::map<Glib::ustring, Glib::ustring>::iterator it = m_data.find(key);

	if(it != m_data.end())
		return true;
	return false;
}
