#ifndef _Document_h
#define _Document_h

/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "SubtitleModel.h"
#include "StyleModel.h"
#include "ScriptInfo.h"
#include "SubtitleModifier.h"
#include "SubtitleView.h"

typedef Glib::RefPtr<SubtitleModel> SubtitleModelPtr;
typedef SubtitleView* SubtitleViewPtr;

/*
 *
 */
class Document
{
public:
	
	/*
	 *	constructeur
	 *	s'il y a une option dans le fichier config [encodings] default=xxxx
	 *	on l'utilise pour choisir le charset, sinon on utilise UTF-8
	 */
	Document();

	/*
	 *	constructeur par copy
	 */
	Document(Document &copy);

	/*
	 *	destructeur
	 */
	~Document();

	/*
	 *	efface les informations :
	 *	 - script info
	 *	 - subtitle model
	 *	 - style model
	 *	
	 *	reinitialize le charset
	 */
	void clear();

	/*
	 *
	 */
	void setFormat(const Glib::ustring &format);

	/*
	 *	SubRip, MicroDVD, ...
	 */
	Glib::ustring getFormat();
	
	/*
	 *
	 */
	void setCharset(const Glib::ustring &charset);

	/*
	 *	retourne le charset (encodage) du document
	 *	a permet de sauvegarder dans le meme encodage qu'a la lecture
	 *	...
	 */
	Glib::ustring getCharset();

	/*
	 *	"Unix", "Windows"
	 */
	void setNewLine(const Glib::ustring &name);

	/*
	 *	"Unix", "Windows"
	 */
	Glib::ustring getNewLine();

	/*
	 *	le nom complet du document 
	 *	ex: /home/toto/subtitle05.ass
	 */
	void setFilename(const Glib::ustring &filename);

	/*
	 *	le nom complet du document 
	 *	ex: /home/toto/subtitle05.ass
	 */
	Glib::ustring getFilename();

	/*
	 *	le nom du document
	 *	si filename = /home/toto/subtitle05.ass
	 *	name = subtitle05.ass
	 */
	void setName(const Glib::ustring &name);

	/*
	 *	le nom du document
	 *	si filename = /home/toto/subtitle05.ass
	 *	name = subtitle05.ass
	 */
	Glib::ustring getName();



	/*
	 *	encodings ""=auto, "UTF-8", ...
	 *	clear permet d'effacer l'ancien model, sinon on ajoute a la suite du document	
	 */
	bool open(const Glib::ustring &filename);

	/*
	 *	format "SubRip", "MicroDVD", "MPL2", ...
	 *	encodings ""=auto, "UTF-8", ...
	 *	rename, change le nom du document (filenameDocument)
	 */
	bool save(const Glib::ustring &filename);


	/*
	 *	retourne le model pour la gestion des sous-titres
	 *	on passe par un model (gtk) pour eviter d'avoir des doublons 
	 *	dans les donnes et donc une grosse consomation memoire (inutile)
	 */
	SubtitleModelPtr get_subtitle_model();

	/*
	 *	retourne le model pour la gestion des styles
	 */
	Glib::RefPtr<StyleModel> get_style_model();

	/*
	 *	retourne les informations sur le script 
	 *	principalement pour SSA/ASS
	 */
	ScriptInfo& get_script_info();

	/*
	 *
	 */
	SubtitleViewPtr get_subtitle_view();


	/*
	 *	affiche un message pour l'utilisateur sur ce document
	 *	statusbar
	 */
	void message(const gchar *format, ...);

	/*
	 *
	 */
	sigc::signal<void, Glib::ustring>& get_signal_message();

	/*
	 *	affiche un message pendant 3 sec 
	 *	statusbar
	 */
	void flash_message(const gchar *format, ...);

	/*
	 *
	 */
	sigc::signal<void, Glib::ustring>& get_signal_flash_message();
	

	/*
	 *	gestion des signaux
	 *
	 *	"document-property-changed" (filename, format...)
	 *	"subtitle-selection-changed"
	 *	"subtitle-time-changed"	
	 */

	/*
	 *
	 */
	sigc::signal<void>& get_signal(const std::string &name);

	/*
 	 *
	 */
	void emit_signal(const std::string &name);


	/*
	 *	Data system
	 */
	
	/*
	 *
	 */
	void set_data(const Glib::ustring &key, const Glib::ustring &value);

	/*
	 *
	 */
	Glib::ustring get_data(const Glib::ustring &key);

	/*
	 *
	 */
	bool has_data(const Glib::ustring &key);

protected:
	bool _open(const Glib::ustring &filename, const Glib::ustring &encoding="", bool clear=true);
	bool _save(const Glib::ustring &filename, const Glib::ustring &format, const Glib::ustring &encoding="", bool clear=true);
	
public:
	Glib::ustring									m_format;
	std::string										m_charset;
	Glib::ustring									m_newline;
	ScriptInfo										m_scriptInfo;
	Glib::RefPtr<SubtitleModel>		m_subtitleModel;
	Glib::RefPtr<StyleModel>			m_styleModel;
	//Glib::RefPtr<NameModel>			m_nameModel;
	SubtitleView*									m_subtitleView;

	// filename movie, ...
	//Glib::ustring filenameDocument;
	Glib::ustring m_uriMovie;

protected:
	Glib::ustring m_name;		// ex: toto.ass
	Glib::ustring m_filename;	// /home/titi/toto.ass

	std::map<std::string, sigc::signal<void> >	m_signal;

	sigc::signal<void, Glib::ustring> m_signal_message;
	sigc::signal<void, Glib::ustring> m_signal_flash_message;

	std::map<Glib::ustring, Glib::ustring> m_data;
};

#endif//_Document_h

